@@ -1,9 +1,6 @@
-import math
-
 import gmsh
-import meshio
-import numpy
 
+from ..helpers import extract_to_meshio
 from .bspline import BSpline
 from .circle_arc import CircleArc
 from .curve_loop import CurveLoop
@@ -45,6 +42,12 @@ class CommonGeometry:
         return self
 
     def __exit__(self, *a):
+        try:
+            # Gmsh >= 4.7.0
+            # https://gitlab.onelab.info/gmsh/gmsh/-/issues/1036
+            gmsh.model.mesh.removeSizeCallback()
+        except AttributeError:
+            pass
         gmsh.finalize()
 
     def synchronize(self):
@@ -157,7 +160,7 @@ class CommonGeometry:
         lateral = [Dummy(*e) for e in out_dim_tags[2:]]
         return top, extruded, lateral
 
-    def revolve(
+    def _revolve(
         self,
         input_entity,
         rotation_axis,
@@ -179,7 +182,6 @@ class CommonGeometry:
             else:
                 assert len(num_layers) == len(heights)
 
-        assert angle < math.pi
         assert len(point_on_axis) == 3
         assert len(rotation_axis) == 3
         out_dim_tags = self.env.revolve(
@@ -197,50 +199,6 @@ class CommonGeometry:
         lateral = [Dummy(*e) for e in out_dim_tags[2:]]
         return top, extruded, lateral
 
-    def twist(
-        self,
-        input_entity,
-        translation_axis,
-        rotation_axis,
-        point_on_axis,
-        angle,
-        num_layers=None,
-        heights=None,
-        recombine=False,
-    ):
-        """Twist (translation + rotation) of any entity along a given translation_axis,
-        around a given rotation_axis, about a given angle.
-        """
-        if isinstance(num_layers, int):
-            num_layers = [num_layers]
-        if num_layers is None:
-            num_layers = []
-            heights = []
-        else:
-            if heights is None:
-                heights = []
-            else:
-                assert len(num_layers) == len(heights)
-
-        assert len(point_on_axis) == 3
-        assert len(rotation_axis) == 3
-        assert len(translation_axis) == 3
-        assert angle < math.pi
-        out_dim_tags = self.env.twist(
-            input_entity.dim_tags,
-            *point_on_axis,
-            *translation_axis,
-            *rotation_axis,
-            angle,
-            numElements=num_layers,
-            heights=heights,
-            recombine=recombine,
-        )
-        top = Dummy(*out_dim_tags[0])
-        extruded = Dummy(*out_dim_tags[1])
-        lateral = [Dummy(*e) for e in out_dim_tags[2:]]
-        return top, extruded, lateral
-
     def translate(self, obj, vector):
         """Translates input_entity itself by vector.
 
@@ -325,6 +283,7 @@ class CommonGeometry:
         order=None,
         # http://gmsh.info/doc/texinfo/gmsh.html#index-Mesh_002eAlgorithm
         algorithm=None,
+        verbose=False,
     ):
         """Return a meshio.Mesh, storing the mesh points, cells, and data, generated by
         Gmsh from the `self`.
@@ -368,6 +327,9 @@ class CommonGeometry:
         if order is not None:
             gmsh.model.mesh.setOrder(order)
 
+        if verbose:
+            gmsh.option.setNumber("General.Terminal", 1)
+
         # set algorithm
         # http://gmsh.info/doc/texinfo/gmsh.html#index-Mesh_002eAlgorithm
         if algorithm:
@@ -375,57 +337,4 @@ class CommonGeometry:
 
         gmsh.model.mesh.generate(dim)
 
-        # extract point coords
-        idx, points, _ = gmsh.model.mesh.getNodes()
-        points = points.reshape(-1, 3)
-        idx -= 1
-        srt = numpy.argsort(idx)
-        assert numpy.all(idx[srt] == numpy.arange(len(idx)))
-        points = points[srt]
-
-        # extract cells
-        elem_types, elem_tags, node_tags = gmsh.model.mesh.getElements()
-        cells = []
-        for elem_type, node_tags in zip(elem_types, node_tags):
-            # `elementName', `dim', `order', `numNodes', `localNodeCoord',
-            # `numPrimaryNodes'
-            num_nodes_per_cell = gmsh.model.mesh.getElementProperties(elem_type)[3]
-            meshio.gmsh.gmsh_to_meshio_type
-            cells.append(
-                meshio.CellBlock(
-                    meshio.gmsh.gmsh_to_meshio_type[elem_type],
-                    node_tags.reshape(-1, num_nodes_per_cell) - 1,
-                )
-            )
-
-        cell_sets = {}
-        for dim, tag in gmsh.model.getPhysicalGroups():
-            name = gmsh.model.getPhysicalName(dim, tag)
-            cell_sets[name] = [[] for _ in range(len(cells))]
-            for e in gmsh.model.getEntitiesForPhysicalGroup(dim, tag):
-                # TODO node_tags?
-                # elem_types, elem_tags, node_tags
-                elem_types, elem_tags, _ = gmsh.model.mesh.getElements(dim, e)
-                assert len(elem_types) == len(elem_tags)
-                assert len(elem_types) == 1
-                elem_type = elem_types[0]
-                elem_tags = elem_tags[0]
-
-                meshio_cell_type = meshio.gmsh.gmsh_to_meshio_type[elem_type]
-                # make sure that the cell type appears only once in the cell list
-                # -- for now
-                idx = []
-                for k, cell_block in enumerate(cells):
-                    if cell_block.type == meshio_cell_type:
-                        idx.append(k)
-                assert len(idx) == 1
-                idx = idx[0]
-                cell_sets[name][idx].append(elem_tags - 1)
-
-            cell_sets[name] = [
-                (None if len(idcs) == 0 else numpy.concatenate(idcs))
-                for idcs in cell_sets[name]
-            ]
-
-        # make meshio mesh
-        return meshio.Mesh(points, cells, cell_sets=cell_sets)
+        return extract_to_meshio()
