@@ -1,9 +1,9 @@
+import math
+
 import gmsh
 
 from .. import common
 from .ball import Ball
-
-# from .boolean import Boolean
 from .box import Box
 from .cone import Cone
 from .cylinder import Cylinder
@@ -41,6 +41,16 @@ class Geometry(common.CommonGeometry):
     def characteristic_length_max(self, val):
         gmsh.option.setNumber("Mesh.CharacteristicLengthMax", val)
 
+    def revolve(self, *args, **kwargs):
+        if len(args) >= 4:
+            angle = args[3]
+        else:
+            assert "angle" in kwargs
+            angle = kwargs["angle"]
+
+        assert angle < 2 * math.pi
+        return super()._revolve(*args, **kwargs)
+
     def add_rectangle(self, *args, mesh_size=None, **kwargs):
         entity = Rectangle(*args, **kwargs)
         if mesh_size is not None:
